SETUP_STATES = {
    "STARTING_SETUP": {
        "bitcoind": "Starting Bitcoin Core Setup...",
        "torbrowser": "Starting Tor setup process...",
    },
    "VERIFY_SIGS": {
        "bitcoind": "Verifying signatures...",
        "torbrowser": "Verifying signatures...",
    },
    "START_SERVICE": {
        "bitcoind": "Starting up Bitcoin Core...",
        "torbrowser": "Starting up Tor daemon...",
    },
    "FAILED": {
        "bitcoind": "Failed to start Bitcoin Core...",
        "torbrowser": "Failed to start Tor daemon...",
    },
}
