{% extends "devhelp/components/devhelp_tab.jinja" %}
{% block title %}address-data{% endblock %}
{% set tab = 'index' %}
{% block content %}

<p>
   This Extension is only useful for Developing and testing purposes.
</p>
<p>
   You can checkout and test the different HTML-Components.
</p>

<ul>
   <li><a href="html/address-data.jinja">address-data</a></li>
   <li><a href="html/address-label.jinja">address-label</a></li>
   <li><a href="html/fee-selection.jinja">fee-selection</a></li>
   <li><a href="html/tx-table.jinja">tx-table</a></li>
   <li><a href="html/address-row.jinja">address-row</a></li>
</ul>

<p>
   Same for some macros.
</p>

<ul>
   <li><a href="html/macro_total_wallet_balances.jinja">total_wallet_balances</a></li>
</ul>


{% endblock %}