<div id="tx_broadcast" class="hidden">
    <h1>{{ _("Transaction is Ready to Broadcast") }}</h1>
    <div class="flex-center flex-column">
        <button id="broadcast_local_btn" class="btn flex-item" style="width: 190px;">{{ _("Send transaction") }}</button>
        <button type="button" id="broadcast_block_explorer_btn" class="btn flex-item {% if specter.chain == 'regtest' %}hidden{% endif %}" style="width: 190px;">{{ _("Send via block explorer") }}</button>
        <button type="button" id="broadcast_blockstream_btn" class="btn flex-item hidden" style="width: 190px;">{{ _("Send via blockstream.info") }}</button>
        <button type="button" id="broadcast_mempool_btn" class="btn flex-item hidden" style="width: 190px;">{{ _("Send via mempool.space") }}</button>
        <button type="button" id="broadcast_blockstream_tor_btn" class="btn flex-item hidden" style="width: 190px;">{{ _("Send via blockstream.info") }}<br>{{ _("over Tor hidden service") }}</button>
        <button type="button" id="broadcast_mempool_tor_btn" class="btn flex-item hidden" style="width: 190px;">{{ _("Send via mempool.space") }}<br>{{ _("over Tor hidden service") }}</button>
        <button type="button" id="broadcast_options_btn" class="btn flex-item hidden" style="width: 190px;">{{ _("Back to options") }}</button>
        <button id="save_psbt_btn" class="btn flex-item" style="width: 190px;">{{ _("Save as pending") }}</button>
        <button id="copy_final_tx_btn" class="btn flex-item" style="width: 190px;">{{ _("Copy raw transaction") }}</button>
    </div>
</div>
