{% from 'components/menu_item.jinja' import menu_item %}

{# 
    swan_menu - Tabs menu to navigate between the swan screens.
    Parameters:
    - active_menuitem: Current active tab. Options: 'general', 'bitcoin_core', 'auth'.
 #}
{% macro swan_menu(active_menuitem) -%}
	<nav class="row collapse-on-mobile">
		{{ menu_item(service.id, 'withdrawals', 'Auto-Withdrawals', active_menuitem, isLeft=true) }}
		{{ menu_item(service.id, 'settings', 'Settings', active_menuitem, isRight=true) }}
		<a href="javascript:void(0);" class="mobile-nav-icon" onclick="toggleMobileNav(this, `{{ url_for('static', filename='img/caret-down.svg') }}`, `{{ url_for('static', filename='img/caret-up.svg') }}`)">
			<img style="width: 36px;" src="{{ url_for('static', filename='img/caret-down.svg') }}"/>
		</a>
	</nav>
{%- endmacro %}
