<div style="display: flex; align-items: center;">
    <style>
        .page-limit-fieldset {
            border: none;
            margin-top: 4px; /*to better vertically align on tx history / utxo list overview*/
        }
        .page-limit-select {
            background-color: var(--cmap-bg-lighter);
            background-repeat: no-repeat;
            appearance: none; /* Remove default arrow */
            background-image: url(/../../static/img/down-arrow.svg);
            background-position: right;
            background-size: 1.3em;
            min-width: 80px;
        }
        .page-limit-option {
            background-color: var(--cmap-bg-lighter);
        }
    </style>
    <span class="page-limit-label">{{ _("Page limit") }}: </span>
    <fieldset class="page-limit-fieldset">
        <select class="page-limit-select">
            <option value="" class="page-limit-option" disabled="disabled">Entries</option>
            <option value="10" class="page-limit-option">10</option>
            <option value="25" selected class="page-limit-option">25</option>
            <option value="50" class="page-limit-option">50</option>
            <option value="100" class="page-limit-option">100</option>
            <option value="0" class="page-limit-option">{{ _("All") }}</option>
        </select>
    </fieldset>
</div>