{% from 'wallet/components/wallet_menu_item.jinja' import wallet_menu_item %}

{# 
    wallet_menu - Tabs menu to navigate between the wallet screens.
    Parameters:
    - active_menuitem: Current active tab. Options: 'transactions', 'addresses', 'receive', 'send', 'settings'.
	- wallet_alias: The wallet alias (used in the URL navigation).
 #}
{% macro wallet_menu(active_menuitem, wallet_alias) -%}
	<nav class="row collapse-on-mobile">
		{{ wallet_menu_item('history', _('Transactions'), wallet_alias, active_menuitem, isLeft=true, id='btn_transactions') }}
		{{ wallet_menu_item('addresses', _('Addresses'), wallet_alias, active_menuitem) }}
		{{ wallet_menu_item('receive', _('Receive'), wallet_alias, active_menuitem, id='btn_receive') }}
		{{ wallet_menu_item('send', _('Send'), wallet_alias, active_menuitem, id='btn_send') }}
		{{ wallet_menu_item('settings', _('Settings'), wallet_alias, active_menuitem, isRight=true, id='btn_settings') }}
		<a href="javascript:void(0);" class="mobile-nav-icon" onclick="toggleMobileNav(this, `{{ url_for('static', filename='img/caret-down.svg') }}`, `{{ url_for('static', filename='img/caret-up.svg') }}`)">
			<img style="width: 36px;" src="{{ url_for('static', filename='img/caret-down.svg') }}" class="svg-white"/>
		</a>
	</nav>
{%- endmacro %}
