class DeviceTypes:
    BITBOX02 = "bitbox02"
    BITCOINCORE = "bitcoincore"
    BITCOINCORE_WATCHONLY = "bitcoincore_watchonly"
    COBO = "cobo"
    COLDCARD = "coldcard"
    ELECTRUM = "electrum"
    ELEMENTSCORE = "elementscore"
    GENERICDEVICE = "other"
    JADE = "jade"
    KEEPKEY = "keepkey"
    LEDGER = "ledger"
    SEEDSIGNER = "seedsigner"
    SPECTERDIY = "specter"
    TREZOR = "trezor"
    KEYSTONE = "keystone"
    PASSPORT = "passport"
