{% from 'wallet/components/wallet_menu_item.jinja' import wallet_menu_item %}

{# 
    send_nav - Tabs menu to navigate between the wallet send New and Pending screens.
    Parameters:
    - active_menuitem: Current active tab.
	- wallet_alias: The wallet alias (used in the URL navigation).
 #}
{% macro send_nav(active_menuitem, wallet_alias) -%}
	<br>
    <nav class="row collapse-on-mobile">
		{{ wallet_menu_item('send_new', _('New'), wallet_alias, active_menuitem, isLeft=true) }}
		{{ wallet_menu_item('send_pending', _('Unsigned'), wallet_alias, active_menuitem) }}
		{{ wallet_menu_item('import_psbt', _('Import'), wallet_alias, active_menuitem, isRight=true) }}
		<a href="javascript:void(0);" class="mobile-nav-icon" onclick="toggleMobileNav(this, `{{ url_for('static', filename='img/caret-down.svg') }}`, `{{ url_for('static', filename='img/caret-up.svg') }}`)">
			<img style="width: 36px;" src="{{ url_for('static', filename='img/caret-down.svg') }}" class="svg-white"/>
		</a>
	</nav>
{%- endmacro %}
