{{ plugin_card('Warden', 'Portfolio Tracker.', "https://github.com/pxsocs/warden/raw/master/warden/static/images/warden_square_logo.png", "https://github.com/pxsocs/warden") }}
{{ plugin_card('Spotbit', 'Price Info Service.', "https://github.com/BlockchainCommons/spotbit/raw/master/images/logos/spotbit_avatar.jpg", "https://github.com/BlockchainCommons/spotbit") }}
{{ plugin_card('Device Blinder', 'Privacy in Collobarative Multisig.', url_for('static', filename='img/ghost_3d.png'), "https://github.com/mflaxman/blind-xpub") }}
{{ plugin_card('Something missing?', 'Add to the wishlist.', url_for('static', filename='img/ghost.png'), "https://github.com/cryptoadvance/specter-desktop/edit/master/src/cryptoadvance/specter/templates/services/components/wishlist.jinja") }}