{% extends "base.jinja" %}

{% block sidebar %}
{% endblock %}

{% block main %}
    <div class="card">
        <h1>{{ _("Login to Specter") }}</h1>
        <form action="{{ url_for('auth_endpoint.login') }}" method="POST" role="form">
            {% if specter.config['auth']['method'] == 'usernamepassword' %}
                <input id="username" class="form-control" placeholder="{{ _('Username') }}" name="username" type="text" value=""><br><br>
            {% endif %}
            <input id="password" class="form-control" placeholder="{{ _('Password') }}" name="password" type="password" value="">
            <input type="hidden" name="next" value="{{ data.next }}">
            <input type="hidden" class="csrf-token" name="csrf_token" value="{{ csrf_token() }}"/>
            <br>
            <br>
            <div class="row">
                <button type="submit" id="login-btn" class="btn">{{ _("Login") }}</button>
            </div>
        </form>
    </div>
{% endblock %}
