{% extends "wallet/components/wallet_tab.jinja" %}
{% set tab = 'addresses' %}
{% block content %}

	<div class="table-holder">

		{% include "includes/services-data.html" %}
		{% include "includes/hwi/hwi.jinja" %}
		{% include "includes/explorer-link.html" %}
		{% include "includes/address-data.html" %}
		{% include "includes/address-row.html" %}
        {% include "includes/addresses-table.html" %}
        
		<addresses-table
			{% if specter.price_check and (specter.alt_rate and specter.alt_symbol) %}
				price="{{ specter.alt_rate }}"
				symbol="{{ specter.alt_symbol }}"
			{% endif %}
			btc-unit="{{ specter.unit }}"
			hide-sensitive-info="{{ specter.hide_sensitive_info | lower }}"
			type="receive"
			wallet="{{ wallet.alias }}"></addresses-table>
	</div>
	<div id="address-popup" class="hidden"></div>
{% endblock %}
